import 'dart:async';

import 'package:angular/angular.dart';
import 'package:angular_components/angular_components.dart';
import 'package:angular_redux_dev_tools/redux_dev_tools_component.dart';
import 'package:redux_dev_tools/redux_dev_tools.dart';

/// [DevToolsStore] for a simple counter app.
final _devToolsStore = new DevToolsStore<int>(counterReducer, initialState: 0);

/// Reducer for a coupter app with increment and decrement actions.
int counterReducer(int state, Object action) {
  if (action == Actions.increment) {
    return state + 1;
  } else if (action == Actions.decrement) {
    return state - 1;
  }

  return state;
}

/// Set of available actions.
enum Actions {
  /// +1
  increment,

  /// -1
  decrement
}

/// A counter app with increment, decrement actions with conditional and asynchronous action options.
@Component(
  selector: 'vanilla-counter',
  styles: const ['material-button { background-color: white; }'],
  template: '''
    <h1>
      <material-button icon (trigger)="decrement()">
        <material-icon icon="exposure_neg_1">decrement</material-icon>
      </material-button>
      {{store?.state}}
      <material-button icon (trigger)="increment()">
        <material-icon icon="exposure_plus_1">increment</material-icon>
      </material-button>
    </h1>

    <div>
      <material-button raised (trigger)="incrementIfOdd()">
        if odd <material-icon icon="exposure_plus_1">increment if odd</material-icon>
      </material-button>    
    
      <material-button raised (trigger)="incrementAsync()">
        async <material-icon icon="exposure_plus_1">increment async</material-icon>
      </material-button>
    </div>

    <br>
    <br>

    <redux-dev-tools [store]="store"></redux-dev-tools>
  ''',
  directives: const [ReduxDevToolsComponent, materialDirectives],
)
class VanillaCounterComponent {
  /// Getter for associated [DevToolsStore].
  DevToolsStore get store => _devToolsStore;

  /// Dispatch a decrement action.
  void decrement() => store.dispatch(Actions.decrement);

  /// Dispatch an increment action
  void increment() => store.dispatch(Actions.increment);

  /// Dispatch an increment action asynchonrously after 1 second.
  void incrementAsync() {
    new Future<Null>.delayed(const Duration(milliseconds: 1000)).then((_) {
      store.dispatch(Actions.increment);
    });
  }

  /// Increment the counter conditionally based on the current value being odd.
  void incrementIfOdd() {
    if (store.state % 2 != 0) {
      store.dispatch(Actions.increment);
    }
  }
}
