@Tags(const ['aot'])
@TestOn('browser')
import 'package:angular_test/angular_test.dart';
import 'package:mockito/mockito.dart';
import 'package:redux_dev_tools/redux_dev_tools.dart';
import 'package:test/test.dart';

import '_test_dependencies.dart';

/// Tests using a real mock [DevToolsStore] object to test dispatching of [DevToolsActionTypes].
void main() {
  DevToolsAction devToolsAction;

  setUpAll(mockStoreSetUpAll);

  setUp(() async {
    await mockStoreSetUp();
    await devToolsPO.clickDropdown();
    devToolsPO = await devToolsFixture.resolvePageObject(DevToolsPO);
    devToolsAction = null;
  });

  test('clicking SAVE should dispatch DevToolsAction.save', () async {
    await devToolsPO.clickSave();
    final verification = verify(devToolsStore.dispatch(captureAny));
    expect(verification.captured, isNotEmpty);

    expect(verification.captured.first, const isInstanceOf<DevToolsAction>());
    devToolsAction = verification.captured.first;

    expect(devToolsAction.type, equals(DevToolsActionTypes.Save));
  });

  test('clicking resetButton should dispatch DevToolsAction.reset', () async {
    await devToolsPO.clickReset();
    final verification = verify(devToolsStore.dispatch(captureAny));
    expect(verification.captured, isNotEmpty);

    expect(verification.captured.first, const isInstanceOf<DevToolsAction>());
    devToolsAction = verification.captured.first;

    expect(devToolsAction.type, equals(DevToolsActionTypes.Reset));
  });

  test('clicking recomputeButton should dispatch DevToolsAction.recompute',
      () async {
    await devToolsPO.clickRecompute();
    final verification = verify(devToolsStore.dispatch(captureAny));
    expect(verification.captured, isNotEmpty);

    expect(verification.captured.first, const isInstanceOf<DevToolsAction>());
    devToolsAction = verification.captured.first;

    expect(devToolsAction.type, equals(DevToolsActionTypes.Recompute));
  });

  tearDown(disposeAnyRunningTest);
}
