@Tags(const ['aot'])
@TestOn('browser')
import 'package:angular_test/angular_test.dart';
import 'package:redux_dev_tools/redux_dev_tools.dart';
import 'package:test/test.dart';

import '_test_dependencies.dart';

/// Tests using a real [DevToolsStore] object to ensure menus and labels are properly populated.
void main() {
  setUp(() async {
    await devToolsStoreSetUp();
  });

  /// Reusable case for testing dropdownLabel.
  void testDropdownLabel() {
    test('dropdownLabel should display current state', () async {
      expect(await devToolsPO.dropdownDisplayed, true);
      expect(await devToolsPO.dropdownLabel, devToolsStore.state.toString());
    });
  }

  /// Resuable case for testing dropdownLength.
  void testDropdownLength() {
    test('dropdownLength should match computedStates.length', () async {
      expect(devToolsPO.dropdownLength,
          devToolsStore.devToolsState.computedStates.length);
    });
  }

  testDropdownLabel();

  group('after clicking,', () {
    setUp(() async {
      await devToolsPO.clickDropdown();
      devToolsPO = await devToolsFixture.resolvePageObject(DevToolsPO);
    });

    testDropdownLength();

    test('header buttons are displayed', () async {
      expect(await devToolsPO.closeButtonDisplayed, true);
      expect(await devToolsPO.saveButtonDisplayed, true);
      expect(await devToolsPO.resetButtonDisplayed, true);
      expect(await devToolsPO.recomputeButtonDisplayed, true);
    });
  });

  group('after state update,', () {
    setUp(() async {
      devToolsStore.dispatch(new IncrementAction());
      devToolsPO = await devToolsFixture.resolvePageObject(DevToolsPO);
    });

    testDropdownLabel();

    group('then after clicking,', () {
      setUp(() async {
        await devToolsPO.clickDropdown();
        devToolsPO = await devToolsFixture.resolvePageObject(DevToolsPO);
      });

      testDropdownLength();

      test('hides dropdown-items when CLOSE is clicked', () async {
        await devToolsPO.clickClose();
        devToolsPO = await devToolsFixture.resolvePageObject(DevToolsPO);
        expect(devToolsPO.dropdownLength, 0);
      });
    });
  });

  tearDown(disposeAnyRunningTest);
}
